/*
    emndl -- exponentially transformed Mandelbrot Set renderer
    Copyright (C) 2011  Claude Heiland-Allen <claudiusmaximus@goto10.org>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <math.h>

int main(int argc, char **argv) {

  if (argc != 3) exit(1);

  int fdi = open(argv[1], O_RDONLY);
  struct stat s;
  if (fstat(fdi, &s)) exit(1);
  int n = s.st_size / sizeof(float);

  int fdo = open(argv[2], O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);

  for (int i = 0; i < n; ++i) {
    float h;
    if (sizeof(float) != read(fdi, &h, sizeof(float))) exit(1);
    float r, g, b;
/*
    float y, u, v, r, g, b;
    if (h > 0) {
      h *= 12;
      y = 0.5 + 0.25 * cosf(3.618 * h);
      u = 0.5 + 0.25 * cosf(h);
      v = 0.5 + 0.25 * sinf(h);
    } else if (h == 0) {
      y = 1;
      u = 0.5;
      v = 0.5;
    } else {
      y = 0;
      u = 0.5;
      v = 0.5;
    }
    r = y             + 1.402 * v - 0.701;
    g = y - 0.344 * u - 0.714 * v + 0.529;
    b = y + 1.772 * u             - 0.886;
*/
    if (h > 0) {
      float v = 0.5 + log2f(h) / 4;
      r = 100 * v;
      g = 149 * v;
      b = 237 * v;
    } else {
      r = 255;
      g = 204;
      b = 51;
    }
    unsigned char rgb[3];
    rgb[0] = fminf(fmaxf(r, 0), 255);
    rgb[1] = fminf(fmaxf(g, 0), 255);
    rgb[2] = fminf(fmaxf(b, 0), 255);
    if (3 != write(fdo, rgb, 3)) exit(1);
  }

  close(fdo);
  close(fdi);

  return 0;
}
