/*
    emndl -- exponentially transformed Mandelbrot Set renderer
    Copyright (C) 2011  Claude Heiland-Allen <claudiusmaximus@goto10.org>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <cmath>
#include <qd/dd_real.h>
#include <qd/qd_real.h>

#include "emndl.h"

using namespace std;

int main(int argc, char **argv) {
  if (argc != 3) exit(1);
  int fdi = open(argv[1], O_RDONLY);
  int fdo = open(argv[2], O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
  int width;
  int oldwidth = 0;
  float *dwell = 0;
  while (sizeof(int) == read(fdi, &width, sizeof(int))) {
    int dbytes = width * sizeof(float);
    if (width != oldwidth) {
      dwell = (float *) realloc(dwell, dbytes);
      oldwidth = width;
    }
    if (dbytes != read(fdi, dwell, dbytes)) exit(1);
    if (dbytes != write(fdo, dwell, dbytes)) exit(1);
    int k = 0;
    for (int i = 0; i < width; ++i) {
      if (dwell[i] < 0) ++k;
    }
    if (k > 0) {
      int prec;
      if (sizeof(int) != read(fdi, &prec, sizeof(int))) exit(1);
      lseek(fdi, k * prec, SEEK_CUR);
    }
  }
  close(fdi);
  close(fdo);
  return 0;
}

